@@ -1,3 +1,4 @@
+from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.plugins import BasePlugin
 from samtranslator.swagger.swagger import SwaggerEditor
 from samtranslator.open_api.open_api import OpenApiEditor
@@ -20,6 +21,7 @@ class DefaultDefinitionBodyPlugin(BasePlugin):
 
         super(DefaultDefinitionBodyPlugin, self).__init__(DefaultDefinitionBodyPlugin.__name__)
 
+    @cw_timer(prefix="Plugin-DefaultDefinitionBody")
     def on_before_transform_template(self, template_dict):
         """
         Hook method that gets called before the SAM template is processed.
